import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vitalapp/architecture/utils/advance_debounce.dart';
import 'package:vitalapp/architecture/utils/prompt_box.dart';
import 'package:vitalapp/architecture/utils/verify_permissions.dart';
import 'package:vitalapp/components/button.dart';
import 'package:vitalapp/components/search_input.dart';
import 'package:vitalapp/managers/interfaces/patient.dart';
import 'package:vitalapp/pages/facial_recognition/controller.dart';
import 'package:vitalapp/pages/facial_recognition/view.dart';
import 'package:vitalapp/pages/home/controller.dart';
import 'package:vitalapp/pages/id_card_scan/controller.dart';
import 'package:vitalapp/pages/id_card_scan/view.dart';
import 'package:vitalapp/pages/medical_checkup_station/registration/controller.dart';
import 'package:vitalapp/pages/medical_checkup_station/registration/widgets/card_reader/view.dart';
import 'package:vitalapp/pages/medical_checkup_station/registration/widgets/form/index.dart';
import 'package:vitalapp/pages/patient/create/widgets/face_result_dialog.dart';
import 'package:vitalapp/pages/widgets/icon_button.dart';

import 'filter_time.dart';

class RegistrationFilter extends GetView<RegistrationController> {
  RegistrationFilter({
    super.key,
    this.isVital = false,
  });

  /// 是否是家医一体机
  final bool? isVital;
  final _patientManager = Get.find<IPatientManager>();

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        _buildInputAndFilter(),
        FittedBox(
          child: Container(
            child: Row(
              children: [
                VIconButton(
                  iconData: Icons.chrome_reader_mode,
                  textString: '读卡识别',
                  voidCallback: () {
                    advanceDebounce(
                      onReadCardClicked,
                      "registration.onReadCardClicked",
                      1500,
                    );
                  },
                ),
                VIconButton(
                  iconData: Icons.perm_contact_cal_rounded,
                  textString: '拍照识别',
                  voidCallback: () {
                    advanceDebounce(
                      onIdCardScanClicked,
                      "registration.onIdCardScanClicked",
                      1500,
                    );
                  },
                ),
                VIconButton(
                  iconData: Icons.sensor_occupied,
                  textString: '人脸识别',
                  voidCallback: () {
                    advanceDebounce(
                      onFaceIdLoginClicked,
                      "registration.onFaceIdLoginClicked",
                      1500,
                    );
                  },
                ),
                VIconButton(
                  iconData: Icons.edit_document,
                  textString: '手动录入',
                  voidCallback: () {
                    advanceDebounce(
                      onManualInputClicked,
                      "registration.onManualInputClicked",
                      1500,
                    );
                  },
                ),
              ],
            ),
          ),
        ),
      ],
    );
  }

  Widget _buildInputAndFilter() {
    return Row(
      children: [
        Container(
          alignment: Alignment.centerLeft,
          margin: const EdgeInsets.all(10.0), // 设置外边距
          padding: const EdgeInsets.all(5.0), // 设置内边距
          decoration: BoxDecoration(
            color: Colors.grey[200], // 设置背景颜色
            borderRadius: BorderRadius.circular(16.0), // 设置圆角边框
          ),
          height: 50,
          width: 400,
          child: VSearchInput(
            textEditingController: TextEditingController(text: ""),
            placeholder: "请输入身份证号码",
            clearable: true,
            onClear: () {},
            onSearch: (value) {
              var now = DateTime.now();
              var startTime = new DateTime(now.year, now.month, now.day);
              var endTime =
                  startTime.add(Duration(hours: 23, minutes: 59, seconds: 59));
              controller.listController.getRegisterInfoPage(
                keyword: value,
                startTime: startTime,
                endTime: endTime,
              );
            },
          ),
        ),
        IconButton(
          onPressed: () {
            Get.dialog(FilterTime(
              onConfirm: (start, end) {
                controller.listController.getRegisterInfoPage(
                  startTime: start,
                  endTime: end,
                );
              },
            ));
          },
          icon: Icon(
            Icons.filter_alt,
            size: 28,
            color: Colors.black,
          ),
        ),
      ],
    );
  }

  /// 读卡逻辑
  void onReadCardClicked() async {
    PatientDTO? result;

    result = await Get.dialog<PatientDTO>(
      const RegistrationCardReaderDialog(),
      barrierDismissible: false,
    );
    if (result != null) {
      controller.formController.confirmSubmit(
        patient: result,
        isVital: isVital!,
      );
    }
  }

  /// 点击人脸识别
  void onFaceIdLoginClicked() async {
    var verifyResult = await VerifyPermissions.verifyCameraPermissions();
    if (!verifyResult) {
      return;
    }
    final FaceRecognitionResult? result = await Get.to<FaceRecognitionResult>(
      () => const FacialRecognitionPage(
        mode: FacialRecognitionMode.faceRecognition,
      ),
    );
    if (result != null && result.success) {
      final patient = result.patientInfo;
      controller.formController.confirmSubmit(
        patient: PatientDTO(
          code: patient.cardNo,
          cardNo: patient.cardNo,
          patientName: patient.patientName,
          birthday: patient.birthday,
          patientGender: patient.patientGender,
          phone: patient.phone,
          patientAddress: patient.patientAddress,
        ),
        isVital: isVital!,
      );
      // final hasConfirmed = await FaceResultDialog.show(patient, true);
      // if (hasConfirmed) {
      //   await _checkinPatient(patient);
      // }
    } else {
      print("识别取消");
    }
  }

  Future<void> _checkinPatient(PatientBaseDTO patient) async {
    final patientDTO = PatientDTO(
      code: patient.cardNo,
      cardNo: patient.cardNo,
      patientName: patient.patientName,
      nationality: patient.nationality,
      patientGender: patient.patientGender,
      birthday: patient.birthday,
      patientAddress: patient.patientAddress,
    );
    await onIdcardInfoIsCreateRecord(patientDTO.code!);

    Get.find<HomeController>().switchNavByName("/patient/detail");
  }

  Future<void> onIdcardInfoIsCreateRecord(
    String cardNo, [
    PatientBaseDTO? patientInfo,
  ]) async {
    PatientDTO? patientInfoDto = await _patientManager.getDetail(cardNo);
    if (patientInfoDto != null) {
      await _patientManager.switchCurrentPatient(patientInfoDto);
      logger.i(
          'create居民 当前居民是：${patientInfoDto.patientName} 居民code：${patientInfoDto.code}');
    } else {
      PromptBox.toast("切换居民失败！");
    }
    if (patientInfo != null) {
      final hasConfirmed = await FaceResultDialog.show(patientInfo, true);
      if (!hasConfirmed) {
        return;
      }

      if (patientInfoDto != null) {
        Get.find<HomeController>().switchNavByName("/patient/detail");
      }
    }
  }

  /// 扫码识别逻辑
  void onIdCardScanClicked() async {
    var verifyResult = await VerifyPermissions.verifyCameraPermissions();
    if (!verifyResult) {
      return;
    }
    final IdCardScanResult? result = await Get.to<IdCardScanResult>(
      () => const IdCardScanPage(),
    );
    if (result != null && result.success) {
      PromptBox.toast("身份证信息识别成功");
      PatientBaseDTO patientBase = result.patientBaseDTO;
      PatientDTO patientInfo = PatientDTO(
        patientName: patientBase.patientName,
        cardNo: patientBase.cardNo,
        nationality: patientBase.nationality,
        birthday: patientBase.birthday,
        patientGender: patientBase.patientGender,
        patientAddress: patientBase.patientAddress,
      );
      controller.formController.confirmSubmit(
        patient: patientInfo,
        isVital: isVital!,
      );
    } else {
      print("识别取消");
    }
  }

  /// 手动输入信息逻辑
  void onManualInputClicked() async {
    controller.patientInfo = PatientDTO();
    final PatientDTO? result = await Get.dialog<PatientDTO>(
      RegistrationFormDialog(patient: controller.patientInfo),
      barrierDismissible: false,
    );
    controller.formController.createResidentInList(
      patient: result!,
      isVital: isVital!,
    );
  }

  Widget _buildbuttonItem({
    required IconData iconData,
    required String title,
    required VoidCallback buttonTap,
    double width = 120,
  }) {
    return Container(
      width: width,
      height: 50,
      margin: EdgeInsets.symmetric(horizontal: 8),
      child: VButton(
        onTap: buttonTap,
        child: Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Icon(iconData, size: 24),
            Text(title, style: TextStyle(fontSize: 20)),
          ],
        ),
      ),
    );
  }
}
