import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vitalapp/architecture/utils/datetime.dart';
import 'package:vitalapp/architecture/utils/prompt_box.dart';
import 'package:vitalapp/components/cell.dart';
import 'package:vitalapp/components/dialog_date.dart';
import 'package:vitalapp/components/dialog_input.dart';
import 'package:vitalapp/components/dialog_select.dart';
import 'package:vitalapp/consts/nations.dart';
import 'package:vitalapp/consts/rpc_enum_labels.dart';
import 'package:vitalapp/helper/id_card_helper.dart';
import 'package:vitalapp/pages/patient_info/widgets/health_infos.dart';

import 'controller.dart';
import 'widgets/detail_infos.dart';

class PatientInfomationPage extends GetView<PatientInfomationController> {
  final bool isNeedSyncAddressCheck;
  final bool isCanEditId;
  final bool isRequiredName;
  final bool isRequiredCard;
  PatientInfomationPage({
    this.isNeedSyncAddressCheck = false,
    this.isCanEditId = false,
    this.isRequiredName = false,
    this.isRequiredCard = false,
  });

  @override
  Widget build(BuildContext context) {
    return VListFormCellGroup(
      children: [
        VListFormCellGroup(
          formTitle: "基本信息",
          children: _buildBaseInfo(),
        ),
        DetailInfomations(),
        HealthInfomations(),
      ],
    );
  }

  List<Widget> _buildBaseInfo() {
    final state = controller.state;
    return [
      Obx(
        () => VListFormCell(
          label: "姓名",
          content: controller.state.name,
          isRequired: isRequiredName,
          onTap: () async {
            final result = await VDialogInput(
              title: "姓名",
              initialValue: controller.state.name,
              placeholder: "请填写姓名",
            ).show();
            if (result != null) {
              controller.state.name = result;
            }
          },
          endIcon: _buildEndIcon(),
        ),
      ),
      Obx(
        () => VListFormCell(
          label: "证件号",
          content: controller.state.cardNo,
          isRequired: isRequiredCard,
          onTap: () async {
            if (isCanEditId) {
              print("点击修改证件号");
              final result = await VDialogInput(
                title: "证件号",
                initialValue: controller.state.cardNo,
                placeholder: "请填写证件号",
              ).show();
              if (result != null && result is String) {
                // 校验身份证号
                bool validate = IdCardHelper.validateIDCard(result);
                if (!validate) {
                  PromptBox.toast("身份证号码格式错误");
                  return;
                }
                controller.state.cardNo = result;
              }
            }
          },
          endIcon: _buildEndIcon(isEdit: isCanEditId),
        ),
      ),
      Obx(
        () => VListFormCell(
          label: "联系电话",
          content: controller.state.phoneNo,
          onTap: () async {
            final result = await VDialogInput(
              title: "联系电话",
              initialValue: controller.state.phoneNo,
              placeholder: "请填写联系电话",
            ).show();
            if (result != null) {
              controller.state.phoneNo = result;
            }
          },
          endIcon: _buildEndIcon(),
        ),
      ),
      Obx(
        () => VListFormCell(
          label: "联系人电话",
          content: controller.state.emergencyPhone,
          onTap: () async {
            final result = await VDialogInput(
              title: "联系人电话",
              initialValue: controller.state.emergencyPhone,
              placeholder: "请填写联系人电话",
            ).show();
            if (result != null) {
              controller.state.emergencyPhone = result;
            }
          },
          endIcon: _buildEndIcon(),
        ),
      ),
      Obx(
        () => VListFormCell(
          label: "性别",
          content: RpcEnumLabels.gender[state.gender],
          onTap: () async {
            final result = await VDialogSelect<GenderEnum, GenderEnum>(
              title: "选择性别",
              source: const [
                GenderEnum.Male,
                GenderEnum.Female,
                GenderEnum.Unknown,
              ],
              valueGetter: (data) => data,
              labelGetter: (data) => RpcEnumLabels.gender[data] ?? "",
              initialValue: state.gender,
            ).show();
            if (result != null) {
              controller.state.gender = result;
            }
          },
          endIcon: _buildEndIcon(),
        ),
      ),
      Obx(
        () => VListFormCell(
          label: "民族",
          content: state.nation,
          onTap: () async {
            final result = await VDialogSelect<String, String>(
              title: "选择民族",
              source: Nations.china,
              valueGetter: (data) => data,
              labelGetter: (data) => data,
              initialValue: state.nation,
            ).show();
            if (result != null) {
              controller.state.nation = result;
            }
          },
          endIcon: _buildEndIcon(),
        ),
      ),
      Obx(
        () => VListFormCell(
          label: "出生日期",
          content: controller.state.birthday != null
              ? DataTimeUtils.formatDateString(controller.state.birthday!)
              : "",
          onTap: () async {
            final result = await VDialogDate(
              title: "设置出生日期",
              initialValue: controller.state.birthday,
              maxValue: DateTime.now(),
            ).show();
            if (result != null) {
              controller.state.birthday = result;
            }
          },
          endIcon: _buildEndIcon(),
        ),
      ),
      Obx(
        () => VListFormCell(
          label: "年龄",
          content: controller.state.age,
          endIcon: _buildEndIcon(isEdit: false),
          onTap: () {},
        ),
      ),
      Obx(
        () => VListFormCell(
          label: "现住地址",
          content: controller.state.address,
          onTap: () async {
            final result = await VDialogInput(
              title: "现住地址",
              initialValue: controller.state.address,
              placeholder: "请填写现住地址",
            ).show();
            if (result != null) {
              controller.state.address = result;
            }
          },
          endIcon: _buildEndIcon(),
        ),
      ),
      if (isNeedSyncAddressCheck)
        Obx(
          () => VListFormCell(
            label: "同步户籍地址到现住地址",
            labelWidth: 250,
            contentWidget: Container(
              child: Switch(
                onChanged: (value) {
                  controller.onSyncAddressCheckChanged(value);
                },
                value: controller.state.isSyncAddresses,
              ),
            ),
          ),
        ),
      Obx(
        () => VListFormCell(
          label: "户籍地址",
          content: controller.state.permanentResidenceAddress,
          onTap: () async {
            final result = await VDialogInput(
              title: "户籍地址",
              initialValue: controller.state.permanentResidenceAddress,
              placeholder: "请填写户籍地址",
            ).show();
            if (result != null) {
              controller.state.permanentResidenceAddress = result;
            }
          },
          endIcon: _buildEndIcon(),
        ),
      ),
      Obx(
        () => VListFormCell(
          label: "建档单位",
          content: controller.state.createdOrgName,
          endIcon: _buildEndIcon(isEdit: false),
          onTap: () {},
        ),
      ),
      Obx(
        () => VListFormCell(
          label: "建档人",
          content: controller.state.createdDoctorName,
          endIcon: _buildEndIcon(isEdit: false),
          onTap: () {},
        ),
      ),
      Obx(
        () => VListFormCell(
          label: "建档时间",
          content: controller.state.createTime,
          endIcon: _buildEndIcon(isEdit: false),
          onTap: () {},
        ),
      ),
    ];
  }

  Widget _buildEndIcon({bool isEdit = true}) {
    return Container(
      margin: EdgeInsets.only(left: isEdit ? 15 : 35),
      child: isEdit ? Icon(Icons.edit_outlined) : SizedBox(),
    );
  }
}
